/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.clientserverapi.topology;

import java.util.Map;
import java.util.Set;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.topology.exception.AttachContainerException;
import org.ow2.petals.clientserverapi.topology.exception.DetachContainerException;
import org.ow2.petals.clientserverapi.topology.exception.DomainIsNotDynamicException;
import org.ow2.petals.clientserverapi.topology.exception.InconsistencyException;
import org.ow2.petals.clientserverapi.topology.exception.InconsistentDomainNameException;
import org.ow2.petals.clientserverapi.topology.exception.InvalidPassphraseException;
import org.ow2.petals.clientserverapi.topology.exception.NoContainerConfigurationProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoContainerNameProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoDomainConfigurationProvidedException;
import org.ow2.petals.clientserverapi.topology.exception.NoSecurityPassPhraseProvidedException;


/**
 * Client interface of the service 'Topology'
 * 
 * @author Christophe DENEUX - Linagora
 */
public interface TopologyService {

    /**
     * Default value of the topology object lock wait time, in seconds
     */
    public static final int DEFAULT_TOPOLOGY_OBJECT_LOCK_WAIT_TIME = 30;

    /**
     * Default value of the start delay of the topology pinger, in milliseconds
     */
    public static final long DEFAULT_TOPOLOGY_PINGER_START_DELAY = 2000;

    /**
     * Default value of the period delay of the topology pinger, in milliseconds
     */
    public static final long DEFAULT_TOPOLOGY_PINGER_PERIOD_DELAY = 5000;

    /**
     * Name and values of the property identifying a configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class TopologyItem {

        private TopologyItem() {
            // NO constructor because it's an utility class
        }

        /**
         * The property name
         */
        public static final String TOPOLOGY_TYPE = "type";

        /**
         * Value indentifying a domain configuration
         */
        public static final String DOMAIN = "domain";

        /**
         * Value indentifying a container configuration
         */
        public static final String CONTAINER = "container";
    }

    /**
     * Names of properties contained into a domain configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class DomainPropertyName {

        private DomainPropertyName() {
            // NO constructor because it's an utility class
        }

        /**
         * Domain key name: domain name
         */
        public static final String CONF_DOMAIN_NAME = "domainName";

        /**
         * Domain key name: domain Description
         */
        public static final String CONF_DOMAIN_DESCRIPTION = "domainDescription";

        /**
         * Domain key name: JNDI Factory
         */
        public static final String CONF_JNDI_FACTORY = "jndiFactory";

        /**
         * Domain key name: JNDI Provider URL
         */
        public static final String CONF_JNDI_PROVIDER_URL = "jndiProviderUrl";

        /**
         * Domain key name: JNDI Security Principal
         */
        public static final String CONF_JNDI_SECURITY_PRINCIPAL = "jndiSecurityPrincipal";

        /**
         * Domain key name: JNDI Security Credentials
         */
        public static final String CONF_JNDI_SECURITY_CREDENTIALS = "jndiSecurityCredentials";

        /**
         * Domain key name: JNDI Pool Size
         */
        public static final String CONF_JNDI_POOL_SIZE = "jndiPoolSize";

        /**
         * Domain key name: JNDI Pool Size
         */
        public static final String CONF_JNDI_BATCH_SIZE = "jndiBatchSize";
        
        /**
         * Domain key name: Domain mode
         */
        public static final String CONF_DOMAIN_MODE = "domainMode";

        /**
         * Domain key name: Shared area implementation
         */
        public static final String CONF_SHARED_AREA_IMPLEMENTATION = "sharedAreaImplementation";
    }

    /**
     * Values of properties contained into the domain configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class DomainPropertyValues {

        private DomainPropertyValues() {
            // NO constructor because it's an utility class
        }

        /**
         * Domain mode: 'static'
         */
        public static final String MODE_STATIC = "static";

        /**
         * Domain mode: 'dynamic'
         */
        public static final String MODE_DYNAMIC = "dynamic";

    }

    /**
     * Names of properties contained into a container configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class ContainerPropertyName {

        private ContainerPropertyName() {
            // NO constructor because it's an utility class
        }

        /**
         * Container key name: Container Name
         */
        public static final String CONF_CONTAINER_NAME = "containerName";

        /**
         * Container key name: Container Description
         */
        public static final String CONF_CONTAINER_DESCRIPTION = "containerDescription";

        /**
         * Container key name: Domain Name
         */
        public static final String CONF_DOMAIN_NAME = "domainName";

        /**
         * Container key name: Container Host Name
         */
        public static final String CONF_HOST = "host";

        /**
         * Container key name: User
         */
        public static final String CONF_USER = "user";

        /**
         * Container key name: Password
         */
        public static final String CONF_PWD = "password";

        /**
         * Container key name: JMX RMI server port
         */
        public static final String CONF_JMX_RMI_PORT = "jmxRMIPort";

        /**
         * Container key name: transport TCP port
         */
        public static final String CONF_TRANSPORT_TCP_PORT = "transportTCPPort";

        /**
         * Container key name: Container state
         */
        public static final String CONF_STATE = "state";
    }

    /**
     * Values of properties contained into the container configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class ContainerPropertyValues {

        private ContainerPropertyValues() {
            // NO constructor because it's an utility class
        }

        /**
         * Container State: Started or Reachable
         */
        public static final String STATE_REACHABLE = "REACHABLE";

        /**
         * Container State: Stopped or Unreachable
         */
        public static final String STATE_UNREACHABLE = "UNREACHABLE";

    }

    /**
     * Default values of properties contained into the container configuration map returned by
     * {@link TopologyService#retrieveTopology(String)}.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public static class ContainerPropertyDefaultValues {

        private ContainerPropertyDefaultValues() {
            // NO constructor because it's an utility class
        }

        /**
         * Default JMX RMI server port
         */
        public static final String CONF_JMX_RMI_PORT = "";

        /**
         * Default transport TCP port
         */
        public static final String CONF_TRANSPORT_TCP_PORT = "";

    }

    /**
     * Retrieve the topology currently used, as collections of containers, domains and domain configurations.
     * 
     * @param securityPassPhrase
     *            Passphrase needed to return sensible information. If <code>null</code>, empty or not matching the one
     *            of the container on which the connection is established:
     *            <ul>
     *            <li>if <code>isSensibleInformationExpected</code> is set to <code>false</code>, no sensible
     *            information will be returned,</li>
     *            <li>if <code>isSensibleInformationExpected</code> is set to <code>true</code>, the exception
     *            {@link InvalidPassphraseException} is thrown.</li>
     *            </ul>
     * @param isSensibleInformationExpected
     *            Drive the processing of this method against an empty or invalid passphrase.
     * @return a {@link Set} of containers, domain configurations in key/value form. An empty property is associated to
     *         an unset parameter.
     * @throws PetalsException
     *             An error occurs
     */
    public Set<Map<String, String>> retrieveTopology(final String securityPassPhrase,
            final boolean isSensibleInformationExpected) throws PetalsException;

    /**
     * <p>
     * Attach the current container from its initial domain to the domain of the given container. To move a container
     * from a distributed topology to a standalone topology composed only of the current container, the API
     * {@link #detachContainer()} must be used.
     * </p>
     * <p>
     * Note 1: As the shared area implementation can be changed, we use a container of the target domain to retrieve the
     * configuration parameters of the shared area, that is simpler than provided them. The shared area configuration
     * parameters are dependent of the shared area implementation: how to define them at Petals CLI level to attach a
     * container ? which names ? how to do when a new shared area implementation is available ?
     * </p>
     * <p>
     * Note 2: At the same time, only one attachment can occurs from the current domain of the container and only one
     * attachment can occurs to the target domain.
     * </p>
     * <p>
     * Attaching the current container consists of:
     * <ol>
     * <li>get the shared area configuration of the domain in which the container will be attached,</li>
     * <li>stopping router and transporters of the container to attach,</li>
     * <li>deregister from the initial topology all endpoints of the container to attach,</li>
     * <li>restart the shared area client to use the shared area of the new domain,</li>
     * <li>register all endpoints of the container to attach on the new shared area,</li>
     * <li>start router and transporters of the container to attach</li>
     * </ol>
     * </p>
     * <p>
     * </p>
     * The current container interacts with the container of its next domain using JMX. JMX connection information are
     * defined in the given container configuration.
     * <p>
     * If the given container is not running, not available or unreachable, the attachment is aborted.
     * </p>
     * <p>
     * About the given container attributes:
     * <ul>
     * <li>if no host is set, <code>localhost</code> is used as hostname of the given container</li>
     * <li>if no port is set, {@link JMXClient.DEFAULT_PORT} is used for the given container</li>
     * <li>the attribute 'domain' is required, otherwise {@linkNoDomainConfigurationProvidedException} is raised</li>
     * <li>the JMX credentials are required if the given container is secured</li>
     * </ul>
     * </p>
     * 
     * @param container
     *            A running container of the domain in which the current container will be added. Not <code>null</code>.
     * @param securityPassPhrase
     *            The security pass-phrase of the given container is required to attach a container. Not
     *            <code>null</code>.
     * 
     * @throws NoContainerConfigurationProvidedException
     *             The container to add is <code>null</code>
     * @throws NoContainerNameProvidedException
     *             The container name is not defined inside the container configuration (<code>container</code>).
     * @throws NoDomainConfigurationProvidedException
     *             The domain name is not defined inside the container configuration (<code>container</code>).
     * @throws DomainIsNotDynamicException
     *             The current or target domain is not <b>dynamic</b>. Using a container in such domain is prohibited.
     * @throws InconsistentNameException
     *             The container name set in the given container configuration is inconsistent with the topology of the
     *             remote container (it does not exist in the target topology)
     * @throws InconsistentDomainNameException
     *             The domain set in the given container configuration is inconsistent with the topology of the remote
     *             container
     * @throws NoSecurityPassPhraseProvidedException
     *             No security pass-phrase provided, or <code>null</code>
     * @throws AttachContainerException
     *             Failed to attach the container to the domain of the given container.
     */
    public void attachContainerTo(final ContainerConfiguration container, final String securityPassPhrase)
            throws NoContainerConfigurationProvidedException, NoContainerNameProvidedException,
            NoDomainConfigurationProvidedException, DomainIsNotDynamicException, InconsistencyException,
            NoSecurityPassPhraseProvidedException, AttachContainerException;

    /**
     * <p>
     * Detach the current container from its domain to put them in a stand-alone topology
     * </p>
     * <p>
     * Note 1: Only one detachment can occurs for the current container at the same time.
     * </p>
     * <p>
     * Detaching the current container consists of:
     * <ol>
     * <li>stopping router and transporters of the container to detach,</li>
     * <li>deregister all endpoints of the container to detach,</li>
     * <li>remove the current container from the topology,</li>
     * <li>set a dummy topology to the container to detach.</li>
     * </ol>
     * </p>
     * <p>
     * 
     * @throws DomainIsNotDynamicException
     *             The current domain is not <b>dynamic</b>.
     * @throws DetachContainerException
     *             Failed to attach the container to the domain of the given container.
     */
    public void detachContainer() throws DomainIsNotDynamicException, DetachContainerException;

    /**
     * Retrieve the shared area client implementation of the domain of the current container
     * 
     * @return The shared area client implementation of the domain. Not null.
     * @throws PetalsException
     *             An error occurs
     */
    public String retrieveSharedAreaCltImplementation() throws PetalsException;

}
