/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.clientserverapi.transport.monitoring;

import java.util.Map;

/**
 * Management Bean of the TCP transporter monitoring service.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public interface TcpTransporterMonitoringService {

    // -----------------------------
    // Messages counters
    // -----------------------------

    /**
     * <p>
     * Get metrics about the number of messages sent to remote containers by the current container, per remote
     * container, since the last start of the transporter.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container name as defined at topology level,</li>
     * <li>the delivery status: PENDING, SUCCEEDED or ERROR.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getOutgoingMessagesCount();

    /**
     * <p>
     * Get metrics about the number of messages received from remote containers by the current container, per remote
     * container, since the last start of the transporter.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container name as IP address,</li>
     * <li>the delivery status: PENDING, SUCCEEDED or ERROR.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getIncomingMessagesCount();

    // -----------------------------
    // Connection counters
    // -----------------------------

    /**
     * <p>
     * Get metrics about the number of outgoing connections established from the current container to remote containers,
     * per remote container, since the last start of the transporter.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container name as defined at topology level.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getEstablishedOutgoingConnections();

    /**
     * <p>
     * Get metrics about the number of outgoing connections currently ACTIVE (in used to send data) from the current
     * container to remote containers, per remote container.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container name as defined at topology level.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getOutgoingActiveConnectionsCurrent();

    /**
     * <p>
     * Get metrics about the max number of outgoing connections that was ACTIVE (in used to send data) in the same time
     * from the current container to remote containers, per remote container.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container name as defined at topology level.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getOutgoingActiveConnectionsMax();

    /**
     * <p>
     * Get metrics about the number of outgoing connections currently IDLE (waiting to send data) from the current
     * container to remote containers, per remote container.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container name as defined at topology level.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getOutgoingIdleConnectionsCurrent();

    /**
     * <p>
     * Get metrics about the max number of outgoing connections that was IDLE (waiting to send data) in the same time
     * from the current container to remote containers, per remote container.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container name as defined at topology level.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getOutgoingIdleConnectionsMax();

    /**
     * <p>
     * Get the max size of the outgoing connection pool, per remote container.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container name as defined at topology level.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getOutgoingConnectionsMax();

    /**
     * <p>
     * Get the min size of the outgoing connection pool, per remote container.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container name as defined at topology level.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getOutgoingConnectionsMin();

    /**
     * <p>
     * Get metrics about the number of incoming connections established from remote containers to the current
     * containers, per remote container, since the last start of the transporter.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container IP address.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getEstablishedIncomingConnections();

    /**
     * <p>
     * Get metrics about the number of incoming connections currently ACTIVE (in used to send data) from remote
     * containers to the current containers, per remote container.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container IP address.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getIncomingActiveConnectionsCurrent();

    /**
     * <p>
     * Get metrics about the max number of incoming connections that was ACTIVE (in used to send data) in the same time
     * from remote containers to the current containers, per remote container.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container IP address.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getIncomingActiveConnectionsMax();

    /**
     * <p>
     * Get metrics about the number of incoming connections currently IDLE (waiting to send data) from remote containers
     * to the current containers, per remote container.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container IP address.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getIncomingIdleConnectionsCurrent();

    /**
     * <p>
     * Get metrics about the max number of incoming connections that was IDLE (waiting to send data) in the same time
     * from remote containers to the current containers, per remote container.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * <ul>
     * <li>the remote container IP address.</li>
     * </ul>
     * </p>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getIncomingIdleConnectionsMax();

}
